/*~FILE: AED_main.c
  (C) Copyright 1999 by SIGNALWARE Corporation.

  Property of: SIGNALWARE Corporation (sm)
               P.O. Box 50225
               Colorado Springs, CO 80949

  ~PRODUCT: AED-xxx - Analog Expansion Daughterboard
  ~VERSION: 3.1

  4/13/99 V1.0 - initial version
  4/17/99 V1.1 - change to application dependent type for buffer
  4/18/99 V1.2 - correction to buffer length in call to program_DMA_channel
          so that it reflects the number of 16bit-words (ESIZE) in the buffer.
  5/20/99 V1.3 - add appl_end
  5/24/99 V1.4 - turn off FPGA and delay before pausing DMA
  5/29/99 V2.0 - remove printout in run cycle
  6/7/99 V2.1 - remove iteration count
  7/28/99 V2.2  - remove type/constant dependence on appl_hdr.h
  8/01/99 V2.3 - Add data to read drop message
  9/15/99 V2.4 - change FPGA control pins  so that
                   DMAC_EN = RSYNC STAT
                   XCNTL0 = start/stop FPGA
                   XSTAT0 = FPGA FIFO error
  9/20/99  V2.5 - minimize data in external memory
  11/5/99  V3.0- add buffer number and code cleanup 
  6/12/00  V3.1 - save fpga code before stopping fpga
  ~!

  ~HEADER:
     Main program for all AED applications employing expansion
     memory interface for access to the daughter board.
  ~!
  ~WARNING:
    This code should not generally be modified for any application.
    If this code is changed other than to correct an error, please
    change the name of the module to avoid confusion.
   ~! */

#ifdef PC
#define cregister /* nil */
#define interrupt /* nil */
#define noprint  0
#else
#define noprint  0 /* set to 1 to remove all printing */
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <common.h>
#include <board.h>
#include <dma.h>
#include <intr.h>
#include "dma_ex.h"
#include "appl_hdr.h"

ApplBlockType dual_data_buffer;
static int buf_count, read_err, bufs_proc;
static int prev_buf_count;
static unsigned int blocks, records, reclen, size, mode;
static unsigned int block_bytes;
const int Bytes_Size[3] = {4,2,1};

#define FPGA_IO_REG (unsigned int *)(0x01500000)

extern void interrupt dma_interrupt(void);
main()
{

  /**********************************************************************/
  /* Initialize EVM                                                     */
  /**********************************************************************/
  if (evm_init())
  {
    evm_led_enable(RED_LED);
    return(ERROR);
  }
  evm_led_disable(RED_LED);
  evm_led_disable(GREEN_LED);

#if !noprint
  printf("\n*** AED TEST PROGRAM STARTED ***\n");
#endif
  evm_codec_disable();
  intr_hook(dma_interrupt, CPU_INT8);
  INTR_ENABLE(CPU_INT8);
  INTR_CLR_FLAG(CPU_INT8);
  INTR_GLOBAL_ENABLE();

  /**********************************************************/
  /* Clear static variables, allocate buffers               */
  /**********************************************************/
  read_err = 0;
  buf_count = 0;
  prev_buf_count = 0;
  bufs_proc = 0;
  appl_parms(&blocks, &records, &reclen, &size, &mode);
  block_bytes = records*reclen*Bytes_Size[size];
  dual_data_buffer.byte = 
        malloc(blocks*block_bytes);
  if (!dual_data_buffer.byte) {
#if !noprint
    printf("Buffer Allocation Error [%dX%d]\n",
         block_bytes, blocks);
#endif
    evm_led_enable(RED_LED);
    delay_msec(500);
    evm_led_disable(RED_LED);
    delay_msec(500);
    evm_led_enable(RED_LED);
    return (ERROR);
  } /* end if */

  /**********************************************************/
  /* Clear the buffers, start DMA, and wait for interrupt   */
  /**********************************************************/
#if !noprint
  printf("Begin application processing (Buffer size = %d)\n",
             block_bytes);
#endif
  if (mode==DMA_FS_MODE)
    program_DMA_channel(dual_data_buffer.byte,
         records*reclen, blocks, size, DMAC_EN_RSYNC_STAT);
  else
    program_DMA_channel(dual_data_buffer.byte,
         records*reclen, blocks, size, DMAC_EN_HIGH);

  appl_init(dual_data_buffer);
  start_DMA_channel(DMA_CH0);

    
    
  /* tell FPGA to start collecting data */
  SET_BIT(DSP_CNTL_ADDR(1), XCNTL0);
  {
    int appl_err, fpga_err;
    unsigned int times;
    ApplBlockType addr;
    appl_err = 0;
    times = 0;

    while (!read_err && !appl_err &&
         !GET_BIT(DSP_STAT_ADDR(1), XSTAT0)) {
      times++;
      if (prev_buf_count != buf_count) {
        addr.byte = &dual_data_buffer.byte
          [(prev_buf_count%blocks)*block_bytes];
        appl_err = appl_process(addr, prev_buf_count);
        prev_buf_count = buf_count;
        bufs_proc++;
      } else {
        appl_err = appl_idle();
      } /* end if */
    } /* end while */

    /* get the FIFO overflow code before stopping */
	fpga_err = GET_BIT(DSP_STAT_ADDR(1), XSTAT0);

    /* tell FPGA to stop collecting data */
    RESET_BIT(DSP_CNTL_ADDR(1), XCNTL0);
    delay_msec(5);
    DMA_PAUSE(DMA_CH0);

  /*********************************************************/
  /* Check for processing errors                           */
  /*********************************************************/
    if (read_err)
      printf("Read drop error %d (bufct=%d, trnct=%d)\n",
            read_err, prev_buf_count,
            GET_FIELD(DMA_XFER_COUNTER_ADDR(DMA_CH0),
                ELEMENT_COUNT, ELEMENT_COUNT_SZ));
    if (appl_err) printf("Application Termination %d\n", appl_err);
    if (fpga_err) printf("FPGA FIFO Overflowed\n");
    printf("Test Loops = %u Bufs Processed = %d Bufs Received = %d\n",
               times, bufs_proc, buf_count);
  } /* end block */


  appl_end();
  printf("OK\n");
  return(OK);
} /* end main */
/*==================================================================*/
/* This code is the interrupt routine; it should not be modified. */
extern void interrupt dma_interrupt(void) {
int frame_comp;

  read_err = GET_BIT(DMA_SECONDARY_CTRL_ADDR(DMA_CH0),RDROP_COND);
  if (read_err) return;
  frame_comp = GET_BIT(DMA_SECONDARY_CTRL_ADDR(DMA_CH0), FRAME_COND);
  if (frame_comp) {
    buf_count++;
    /* reset the frame cond to allow the next interrupt */
    RESET_BIT(DMA_SECONDARY_CTRL_ADDR(DMA_CH0), FRAME_COND);
  } else read_err = -1;
} /* end dma_interrupt */
/*==================================================================*/
